"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BigCommerceManagementApiPlatform = void 0;
const tslib_1 = require("tslib");
/* eslint-disable @typescript-eslint/strict-boolean-expressions */
/* eslint-disable @typescript-eslint/camelcase */
const error_handler_1 = require("../../../../utils/error-handler");
const checkout_error_1 = require("../../../../utils/checkout-error");
const api_1 = require("./api");
const validations_1 = require("./validations");
const map_line_item_1 = require("./utils/map-line-item");
const map_checkout_1 = require("./utils/map-checkout");
const map_shipping_address_1 = require("./utils/map-shipping-address");
const map_consignment_line_item_1 = require("./utils/map-consignment-line-item");
class BigCommerceManagementApiPlatform {
    constructor(config) {
        this.cartKey = `cartId-${config.storeId || ''}`;
        this.storeName = config.storeName || '';
        this.cartId = typeof window !== 'undefined' ? localStorage.getItem(this.cartKey) : null;
    }
    setCartId(cartId) {
        if (cartId) {
            localStorage.setItem(this.cartKey, cartId);
        }
        else {
            localStorage.removeItem(this.cartKey);
        }
        this.cartId = cartId;
    }
    updateCustomerIdInCart() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                if (!this.cartId)
                    throw 'Missing Cart Id';
                const url = api_1.getCartPath(this.cartId);
                yield api_1.api.put(url, {});
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Add customer id to cart failed.', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    getCheckout() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const url = api_1.getCheckoutPath(this.cartId);
            const response = yield api_1.api.get(url);
            if ((response === null || response === void 0 ? void 0 : response.status) === 404) {
                this.setCartId(null);
                return this.getCart();
            }
            return map_checkout_1.mapCheckoutToCart(Object.assign(Object.assign({}, response), { cart: Object.assign(Object.assign({}, response.cart), { redirect_urls: {
                        checkout_url: `/checkout?cartId=${this.cartId}`,
                    } }) }));
        });
    }
    getCart() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const url = api_1.getCartPath(this.cartId);
                if (!this.cartId) {
                    const data = yield api_1.api.post(url, { line_items: [] });
                    if (!(data === null || data === void 0 ? void 0 : data.id))
                        throw 'Missing Cart Id';
                    this.setCartId(data.id);
                    yield this.updateCustomerIdInCart();
                }
                return this.getCheckout();
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Getting cart failed.', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    addItemsToCart(items) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                validations_1.validateItems(items);
                const url = api_1.getCartsPath(this.cartId);
                const data = yield api_1.api.post(url, {
                    line_items: items.map(map_line_item_1.mapLineItem),
                });
                if (!(data === null || data === void 0 ? void 0 : data.id))
                    throw 'Missing Cart Id';
                if (!this.cartId) {
                    this.setCartId(data.id);
                    yield this.updateCustomerIdInCart();
                }
                return this.getCheckout();
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Adding items to cart failed', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    updateItemsInCart(items) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                validations_1.validateLineItems(items);
                const cartId = this.cartId;
                if (!cartId)
                    throw 'Missing Cart Id';
                for (let i = 0; i < items.length; i++) {
                    const lineItem = map_line_item_1.mapLineItem(items[i]);
                    const url = api_1.getItemPath(cartId, lineItem.id);
                    yield api_1.api.put(url, {
                        line_item: lineItem,
                    });
                }
                return this.getCheckout();
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Updating items in cart failed', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    removeItemsFromCart(itemIds) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                validations_1.validateLineItemIds(itemIds);
                const cartId = this.cartId;
                if (!cartId)
                    throw 'Missing Cart Id';
                const lineItemIds = itemIds;
                const urls = lineItemIds.map((lineItemId) => api_1.getItemPath(cartId, lineItemId));
                for (let i = 0; i < lineItemIds.length; i++) {
                    const data = yield api_1.api.delete(urls[i]);
                    // There is an empty response when the last lineItem has been removed
                    if (!data) {
                        this.setCartId(null);
                        return this.getCart();
                    }
                }
                return this.getCheckout();
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Removing items from cart failed', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    addShippingAddress(props) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                validations_1.validateAddShippingAddressProps(props);
                if (!this.cartId)
                    throw 'Missing Cart Id';
                const { lineItems, shippingAddress } = props;
                const url = api_1.getConsignmentsPath(this.cartId);
                yield api_1.api.post(url, [
                    {
                        line_items: lineItems.map(map_consignment_line_item_1.mapConsignmentLineItem),
                        shipping_address: map_shipping_address_1.mapShippingAddress(shippingAddress),
                    },
                ]);
                return this.getCart();
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Adding shipping address failed', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    updateShippingAddress(props) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                validations_1.validateUpdateShippingAddressProps(props);
                if (!this.cartId)
                    throw 'Missing Cart Id';
                const { consignmentId, lineItems, shippingAddress } = props;
                const url = api_1.getConsignmentsPath(this.cartId, consignmentId);
                yield api_1.api.put(url, {
                    line_items: lineItems.map(map_consignment_line_item_1.mapConsignmentLineItem),
                    shipping_address: map_shipping_address_1.mapShippingAddress(shippingAddress),
                });
                return this.getCart();
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Updating shipping address failed', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    getProductTrackingData(item) {
        validations_1.validateItem(item);
        return {
            id: item.id,
            quantity: item.quantity,
            brand: this.storeName,
            variantId: item['variantId'],
            name: item['name'],
            price: item['listPrice'],
        };
    }
    getItemIdInPlatformSpecificFormat(itemId) {
        if (typeof itemId === 'number')
            return itemId;
        const asNumber = Number(itemId);
        if (!isNaN(asNumber))
            throw new Error('itemId is not type of number and cannot be converted to a number');
        return asNumber;
    }
}
exports.BigCommerceManagementApiPlatform = BigCommerceManagementApiPlatform;
