"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateUpdateShippingAddressProps = exports.validateAddShippingAddressProps = exports.validateLineItems = exports.validateItems = exports.validateItem = exports.validateLineItemIds = void 0;
const isString = (value) => typeof value === 'string';
const isObject = (value) => typeof value === 'object' && value !== null;
const isNumber = (id) => {
    return typeof id === 'number';
};
const validateLineItemIds = (ids) => {
    if (new Set(ids).size !== ids.length)
        throw new Error('There cannot be duplicate line item ids.');
    if (!ids.every(isString))
        throw new Error('All items should have a "lineItemId" property of type string');
};
exports.validateLineItemIds = validateLineItemIds;
const validateItem = (item) => {
    if (!item.id)
        throw new Error('Please provide the "id" property for every item.');
    if (!isNumber(item.id))
        throw new Error('All items should have a "id" property of type number.');
    if (!item.quantity)
        throw new Error('Please provide the "quantity" property for every item.');
    return item;
};
exports.validateItem = validateItem;
const validateItems = (items) => {
    items.forEach(validateItem);
};
exports.validateItems = validateItems;
const validateLineItem = (item) => {
    if (!item['lineItemId'])
        throw new Error('Please provide the "lineItemId" property for every item.');
    if (!isString(item['lineItemId']))
        throw new Error('All items should have a "lineItemId" property of type string.');
    return item;
};
const validateLineItems = (items) => {
    for (const item of items)
        validateLineItem(validateItem(item));
};
exports.validateLineItems = validateLineItems;
const getNotAllowedProperties = (object, availableProperties) => Object.keys(object)
    .filter((key) => !availableProperties.includes(key))
    .join(', ');
const validateProps = ({ props, actionName, availableProps, }) => {
    if (!isObject(props))
        throw new Error(`${actionName} properties are not type of object.`);
    if (!Object.keys(props).length)
        throw new Error(`${actionName} properties are empty object.`);
    const notAllowedProperty = getNotAllowedProperties(props, availableProps);
    if (notAllowedProperty)
        throw new Error(`${actionName} properties "${notAllowedProperty}" are not allowed.`);
};
const availableShippingAddressProps = [
    'countryCode',
    'city',
    'state',
    'stateCode',
    'email',
    'firstName',
    'lastName',
    'address1',
    'postalCode',
    'phone',
];
const validateShippingAddress = (shippingAddress) => {
    if (!isObject(shippingAddress))
        throw new Error(`ShippingAddress is not type of object.`);
    if (!Object.keys(shippingAddress).length)
        throw new Error(`ShippingAddress is empty object.`);
    const notAllowedProperty = getNotAllowedProperties(shippingAddress, availableShippingAddressProps);
    if (notAllowedProperty)
        throw new Error(`ShippingAddress properties "${notAllowedProperty}" are not allowed.`);
};
const validateAddShippingAddressProps = (props) => {
    validateProps({
        props,
        actionName: 'AddShippingAddress',
        availableProps: ['lineItems', 'shippingAddress'],
    });
    const { lineItems, shippingAddress } = props;
    if (!Array.isArray(lineItems))
        throw new Error(`AddShippingAddress property "lineItems" is not type of array.`);
    if (!lineItems.length)
        throw new Error(`AddShippingAddress property "lineItems" is empty array.`);
    validateShippingAddress(shippingAddress);
};
exports.validateAddShippingAddressProps = validateAddShippingAddressProps;
const validateUpdateShippingAddressProps = (props) => {
    validateProps({
        props,
        actionName: 'UpdateShippingAddress',
        availableProps: ['consignmentId', 'lineItems', 'shippingAddress'],
    });
    const { consignmentId, lineItems, shippingAddress } = props;
    if (!isString(consignmentId))
        throw new Error(`UpdateShippingAddress property "consignmentId" is not type of string`);
    if (!consignmentId)
        throw new Error(`UpdateShippingAddress property "consignmentId" is empty string`);
    if (!Array.isArray(lineItems))
        throw new Error(`UpdateShippingAddress property "lineItems" is not type of array.`);
    if (!lineItems.length)
        throw new Error(`UpdateShippingAddress property "lineItems" is empty array.`);
    validateShippingAddress(shippingAddress);
};
exports.validateUpdateShippingAddressProps = validateUpdateShippingAddressProps;
