"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ShopifyAjaxPlatform = void 0;
const tslib_1 = require("tslib");
/* eslint-disable @typescript-eslint/strict-boolean-expressions */
/* eslint-disable unicorn/no-array-reduce */
const types_1 = require("./types");
const checkout_error_1 = require("../../../../utils/checkout-error");
const api_1 = require("./api");
const validations_1 = require("./validations");
const error_handler_1 = require("../../../../utils/error-handler");
const map_cart_1 = require("./utils/map-cart");
const id_helpers_1 = require("../utils/id-helpers");
const get_is_change_item_request_1 = require("./utils/get-is-change-item-request");
class ShopifyAjaxPlatform {
    constructor(config, meta) {
        this.storeDomain = config.storeDomain;
        this.storeName = meta.storeName;
    }
    fetchProduct(handle) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                validations_1.validateHandle(handle);
                const product = yield api_1.api.get(api_1.getProductsPath(handle));
                return product;
            }
            catch (error) {
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    fetchProductVariant(variantId) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const externalId = this.getItemIdInPlatformSpecificFormat(variantId);
                const productVariant = yield api_1.api.get(api_1.getVariantsPath(externalId));
                return productVariant;
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Fetching product variant failed.', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    addItemsToCart(items) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const itemsWithExternalIds = items.map((_a) => {
                    var { id } = _a, rest = tslib_1.__rest(_a, ["id"]);
                    return (Object.assign(Object.assign({}, rest), { id: this.getItemIdInPlatformSpecificFormat(id) }));
                });
                validations_1.validateItemsWithIds(itemsWithExternalIds);
                yield api_1.api.post(api_1.ADD_ITEMS_PATH, { items: itemsWithExternalIds });
                const cart = yield api_1.api.get(api_1.GET_CART_PATH);
                return map_cart_1.mapCart(cart, { storeDomain: this.storeDomain });
            }
            catch (error) {
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    updateItemsInCart(items) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                // Depending if user sent 'line' property or 'id' we are determing should we make /cart/change.js
                // (only one item can be change at the time) or /cart/update.js (multiple items can be updated in same time)
                // request to Shopify. Changing item with 'line' is used when there are multile items with same 'id'.
                const firstItem = items[0];
                validations_1.validateItem(firstItem);
                const isChangeItemRequest = get_is_change_item_request_1.getIsChangeItemRequest(firstItem);
                if (isChangeItemRequest) {
                    yield api_1.api.post(api_1.CHANGE_ITEM_PATH, firstItem);
                }
                else {
                    const itemsWithExternalIds = items.map((_a) => {
                        var { id } = _a, rest = tslib_1.__rest(_a, ["id"]);
                        return (Object.assign(Object.assign({}, rest), { id: this.getItemIdInPlatformSpecificFormat(id) }));
                    });
                    validations_1.validateItemsWithIds(itemsWithExternalIds);
                    const updates = itemsWithExternalIds.reduce((acc, item) => {
                        acc[item.id] = item.quantity;
                        return acc;
                    }, {});
                    yield api_1.api.post(api_1.UPDATE_ITEMS_PATH, { updates });
                }
                const cart = yield api_1.api.get(api_1.GET_CART_PATH);
                return map_cart_1.mapCart(cart, { storeDomain: this.storeDomain });
            }
            catch (error) {
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    removeItemsFromCart(itemIds) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const externalIds = itemIds.map((id) => this.getItemIdInPlatformSpecificFormat(id));
                validations_1.validateIds(externalIds);
                // Set all items to have quantity 0, which will remove them from the cart
                const updates = externalIds.reduce((acc, id) => {
                    acc[id] = 0;
                    return acc;
                }, {});
                yield api_1.api.post(api_1.UPDATE_ITEMS_PATH, { updates });
                const cart = yield api_1.api.get(api_1.GET_CART_PATH);
                return map_cart_1.mapCart(cart, { storeDomain: this.storeDomain });
            }
            catch (error) {
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    updateCart({ items, note, attributes }) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                let updates;
                if (items === null || items === void 0 ? void 0 : items.length) {
                    const itemsWithExternalIds = items.map((_a) => {
                        var { id } = _a, rest = tslib_1.__rest(_a, ["id"]);
                        return (Object.assign(Object.assign({}, rest), { id: this.getItemIdInPlatformSpecificFormat(id) }));
                    });
                    validations_1.validateItemsWithIds(itemsWithExternalIds);
                    updates = itemsWithExternalIds.reduce((list, item) => {
                        return Object.assign(Object.assign({}, list), { [item.id]: item.quantity });
                    }, {});
                }
                yield api_1.api.post(api_1.UPDATE_ITEMS_PATH, { updates, note, attributes });
                const cart = yield api_1.api.get(api_1.GET_CART_PATH);
                return map_cart_1.mapCart(cart, { storeDomain: this.storeDomain });
            }
            catch (error) {
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    getCart() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const cart = yield api_1.api.get(api_1.GET_CART_PATH);
                return map_cart_1.mapCart(cart, { storeDomain: this.storeDomain });
            }
            catch (error) {
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    getProductTrackingData(item) {
        if (!types_1.isAjaxItem(item))
            throw new checkout_error_1.CheckoutError('Not valid "item" provided to getProductTrackingData method.');
        return {
            id: item.id,
            variantId: item.variant_id,
            name: item.title,
            brand: this.storeName,
            quantity: item.quantity,
            price: item.price || 0,
        };
    }
    getItemIdInPlatformSpecificFormat(itemId) {
        return id_helpers_1.mapItemIdToExternalId(itemId);
    }
}
exports.ShopifyAjaxPlatform = ShopifyAjaxPlatform;
