"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = void 0;
const tslib_1 = require("tslib");
const constants_1 = require("./constants");
const queries_1 = require("./queries");
const handle_fetch_1 = require("./utils/handle-fetch");
const mappings_1 = require("./mappings");
const process_cart_mutation_result_1 = require("./utils/process-cart-mutation-result");
const get_graph_ql_api_base_url_1 = require("./utils/get-graph-ql-api-base-url");
class Client {
    constructor(config) {
        const { storefrontAccessToken, platformDomain, storeDomain } = config;
        this.platformDomain = platformDomain;
        this.storeDomain = storeDomain;
        const baseApiUrl = get_graph_ql_api_base_url_1.getGraphQLApiBaseUrl(this.storeDomain, this.platformDomain);
        this.apiUrl = `${baseApiUrl}/api/${constants_1.STOREFRONT_API_VERSION}/graphql`;
        this.defaultHeaders = {
            'X-Shopify-Storefront-Access-Token': storefrontAccessToken,
            'Accept-Language': '*',
            'Content-Type': 'application/json',
            Accept: 'application/json',
        };
    }
    fetchProductByHandle(handle) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const query = queries_1.getProductByHandleQuery({ handle });
            const { productByHandle } = yield this.post(query);
            return yield mappings_1.mapProductGraphQlTypeToProduct(productByHandle, this);
        });
    }
    fetchProduct(id) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const query = queries_1.getProductNodeQuery({ id });
            const { node } = yield this.post(query);
            return yield mappings_1.mapProductGraphQlTypeToProduct(node, this);
        });
    }
    createCart(input) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const mutation = queries_1.getCreateCartMutation({ input: input || {} });
            const { cartCreate } = yield this.post(mutation);
            const cart = process_cart_mutation_result_1.processCartMutationResult(cartCreate);
            return mappings_1.mapCartGraphQlTypeToShopifyStorefrontCart(cart, this);
        });
    }
    fetchCart(id) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const query = queries_1.getCartQuery({ id });
            const { cart } = yield this.post(query);
            return mappings_1.mapCartGraphQlTypeToShopifyStorefrontCart(cart, this);
        });
    }
    addCartLines(cartId, lines) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const mutation = queries_1.getAddCartLinesMutation({ cartId, lines });
            const { cartLinesAdd: result } = yield this.post(mutation);
            const cart = process_cart_mutation_result_1.processCartMutationResult(result);
            return mappings_1.mapCartGraphQlTypeToShopifyStorefrontCart(cart, this);
        });
    }
    removeCartLines(cartId, lineIds) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const mutation = queries_1.getRemoveCartLinesMutation({ cartId, lineIds });
            const { cartLinesRemove: result } = yield this.post(mutation);
            const cart = process_cart_mutation_result_1.processCartMutationResult(result);
            return mappings_1.mapCartGraphQlTypeToShopifyStorefrontCart(cart, this);
        });
    }
    updateCartLines(cartId, lines) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const mutation = queries_1.getUpdateCartLinesMutation({ cartId, lines });
            const { cartLinesUpdate: result } = yield this.post(mutation);
            const cart = process_cart_mutation_result_1.processCartMutationResult(result);
            return mappings_1.mapCartGraphQlTypeToShopifyStorefrontCart(cart, this);
        });
    }
    fetchProductVariant(id) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const query = queries_1.getProductVariantQuery({ id });
            const { node } = yield this.post(query);
            return mappings_1.mapProductVariantWithProductGraphQlTypeToShopifyStorefrontProductVariantWithProduct(node, this);
        });
    }
    fetchCartLinesPage(cartId, cursor, pageSize = constants_1.DEFAULT_PAGE_SIZE) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const query = queries_1.getCartLinesPageQuery({ cartId, pageSize, cursor });
            const { cart: { lines }, } = yield this.post(query);
            return lines;
        });
    }
    fetchProductImagesPage(productId, cursor, pageSize = constants_1.DEFAULT_PAGE_SIZE) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const query = queries_1.getProductImagesPageQuery({ productId, pageSize, cursor });
            const { node: { images }, } = yield this.post(query);
            return images;
        });
    }
    fetchProductVariantsPage(productId, cursor, pageSize = constants_1.DEFAULT_PAGE_SIZE) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const query = queries_1.getProductVariantsQuery({ productId, pageSize, cursor });
            const { node: { variants }, } = yield this.post(query);
            return variants;
        });
    }
    fetchPresentmentPricesPage(productVariantId, cursor, pageSize = constants_1.DEFAULT_PAGE_SIZE) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const query = queries_1.getPresentmentPricesPageQuery({ productVariantId, pageSize, cursor });
            const { node: { presentmentPrices }, } = yield this.post(query);
            return presentmentPrices;
        });
    }
    fetchCurrencies() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const query = queries_1.getCurrenciesQuery();
            const { shop: { paymentSettings: { enabledPresentmentCurrencies }, }, } = yield this.post(query);
            return enabledPresentmentCurrencies;
        });
    }
    post(queryOrMutation, headers) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const result = yield handle_fetch_1.handleFetch(this.apiUrl, {
                headers: Object.assign(Object.assign({}, this.defaultHeaders), (headers || {})),
                method: 'POST',
                body: queryOrMutation,
            });
            return result;
        });
    }
}
exports.Client = Client;
