"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ShopifyStorefrontPlatform = void 0;
const tslib_1 = require("tslib");
/* eslint-disable @typescript-eslint/no-explicit-any */
/* eslint-disable no-restricted-syntax */
/* eslint-disable @typescript-eslint/strict-boolean-expressions */
const client_1 = require("./client");
const checkout_error_1 = require("../../../../utils/checkout-error");
const error_handler_1 = require("../../../../utils/error-handler");
const map_id_to_merchandise_id_1 = require("./utils/map-id-to-merchandise-id");
const shared_checkout_1 = require("./utils/shared-checkout");
const id_helpers_1 = require("../utils/id-helpers");
const map_cart_1 = require("./utils/map-cart");
const types_1 = require("./types");
const validations_1 = require("./validations");
class ShopifyStorefrontPlatform {
    constructor(config, meta) {
        this.client = new client_1.Client(config);
        this.storeDomain = config.storeDomain;
        this.storeName = meta.storeName;
        this.cartKey = `cartId-${meta.storeId}`;
        this.sharedCheckoutDomains = meta.sharedCheckoutDomains || [];
        this.isSharedCheckout = this.sharedCheckoutDomains.length > 0;
    }
    /**
     *  Returns a `cartId`.
     *
     * Gets a `cartId` from either:
     * 1. Checking `localStorage` of the current site.
     * 2. Loading `sharedCheckoutDomains` to see if they have one in `storage`.
     * 3. Executing `createCart`, which will generate a new Shopify checkout `cartId`.
     *
     * @returns `cartId`
     */
    getCartId() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const cartId = localStorage.getItem(this.cartKey);
                if (cartId !== null) {
                    const cart = yield this.client.fetchCart(cartId);
                    if (cart)
                        return cartId;
                }
            }
            catch (_a) { }
            const cart = yield this.createCart();
            return cart.id;
        });
    }
    fetchProduct(id) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                if (typeof id === 'string' && !id_helpers_1.isStorefrontId(id)) {
                    return this.client.fetchProductByHandle(id);
                }
                const storefrontId = this.getItemIdInPlatformSpecificFormat(id);
                return this.client.fetchProduct(storefrontId);
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Fetching product by id failed.', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    fetchProductVariant(id) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const storefrontId = this.getItemIdInPlatformSpecificFormat(id, 'ProductVariant');
                return this.client.fetchProductVariant(storefrontId);
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Fetching product variant by id failed.', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    fetchCurrencies() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                return this.client.fetchCurrencies();
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Fetching currencies failed.', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    selectCurrency(currencyWithItems) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                validations_1.validateSelectCurrencyProps(currencyWithItems);
                const { currency, lineItems } = currencyWithItems;
                return this.createCart({
                    lineItems,
                    presentmentCurrencyCode: currency,
                });
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Selecting currency failed.', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    createCart(input) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const cart = yield this.client.createCart(input);
                if (!cart)
                    throw new Error('No cart retrieved from API.');
                try {
                    localStorage.setItem(this.cartKey, String(cart.id));
                    if (this.isSharedCheckout) {
                        // set cartId on shared checkout storage
                        shared_checkout_1.cookieStorage.setItem(this.cartKey, String(cart.id));
                        shared_checkout_1.syncCartIdWithSharedDomains(this.sharedCheckoutDomains, String(cart.id));
                    }
                }
                catch (_a) { }
                return map_cart_1.mapCart(cart, { storeDomain: this.storeDomain });
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Creating cart failed.', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    addItemsToCart(items) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const cartId = yield this.getCartId();
            try {
                const itemsWithStorefrontIds = items.map((_a) => {
                    var { id } = _a, rest = tslib_1.__rest(_a, ["id"]);
                    return (Object.assign(Object.assign({}, rest), { id: this.getItemIdInPlatformSpecificFormat(id, 'ProductVariant') }));
                });
                const lineItems = map_id_to_merchandise_id_1.mapIdToMerchandiseId(itemsWithStorefrontIds);
                validations_1.validateItems(itemsWithStorefrontIds);
                const cart = yield this.client.addCartLines(cartId, lineItems);
                if (!(cart === null || cart === void 0 ? void 0 : cart.id))
                    return this.createCart();
                return map_cart_1.mapCart(cart, { storeDomain: this.storeDomain });
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Adding items to cart failed.', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    updateItemsInCart(items) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const cartId = yield this.getCartId();
            try {
                const itemsWithStorefrontIds = items.map((_a) => {
                    var { id } = _a, rest = tslib_1.__rest(_a, ["id"]);
                    return (Object.assign(Object.assign({}, rest), { id: this.getItemIdInPlatformSpecificFormat(id) }));
                });
                validations_1.validateItems(itemsWithStorefrontIds);
                const cart = yield this.client.updateCartLines(cartId, itemsWithStorefrontIds);
                if (!(cart === null || cart === void 0 ? void 0 : cart.id))
                    return this.createCart();
                return map_cart_1.mapCart(cart, { storeDomain: this.storeDomain });
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Updating items in cart failed.', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    removeItemsFromCart(itemIds) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const cartId = yield this.getCartId();
            try {
                validations_1.validateIds(itemIds);
                const cart = yield this.client.removeCartLines(cartId, itemIds);
                if (!(cart === null || cart === void 0 ? void 0 : cart.id))
                    return this.createCart();
                return map_cart_1.mapCart(cart, { storeDomain: this.storeDomain });
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Removing items from cart failed.', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    getCart() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const cartId = yield this.getCartId();
            try {
                const cart = yield this.client.fetchCart(cartId);
                if (!(cart === null || cart === void 0 ? void 0 : cart.id))
                    return this.createCart();
                return map_cart_1.mapCart(cart, { storeDomain: this.storeDomain });
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Getting cart failed.', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    getProductTrackingData(item) {
        if (!types_1.isStorefrontItem(item))
            throw new checkout_error_1.CheckoutError('Not valid "item" provided to getProductTrackingData method.');
        if (item.variant) {
            return {
                id: item.variant.sku || item.id.toString(),
                name: item.title,
                brand: this.storeName,
                variant: item.variant.title || '',
                price: parseFloat(item.variant.price || '0'),
                compareAtPrice: parseFloat(item.variant.compareAtPrice || '0'),
                quantity: item.quantity,
            };
        }
        return {
            id: String(item.id),
            name: item.title,
            brand: this.storeName,
            quantity: item.quantity,
            price: 0,
        };
    }
    getItemIdInPlatformSpecificFormat(itemId, productType = 'Product') {
        if (id_helpers_1.isStorefrontId(itemId))
            return itemId;
        return {
            Product: id_helpers_1.mapItemIdToProductStorefrontId(itemId),
            ProductVariant: id_helpers_1.mapItemIdToProductVariantStorefrontId(itemId),
        }[productType];
    }
}
exports.ShopifyStorefrontPlatform = ShopifyStorefrontPlatform;
