"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mapItemIdToExternalId = exports.mapItemIdToProductVariantStorefrontId = exports.mapItemIdToProductStorefrontId = exports.isExternalId = exports.isStorefrontId = void 0;
/**
 * Shopify GID Notes
 *
 * 08/Feb 2022 CartLines from CartAPI do not match standard format
 * ---------------------------------------------------------------
 * Normal Format: gid://shopify/(Product | ProductVariant | CheckoutLineItem)/<externalId>
 *
 * CartLine Format: gid://shopify/CartLine/<uuid>
 *
 * We can no longer map CartLine Ids to REST compatible externalIds
 */
const shopifyGidUriRegExp = /^(gid):\/\/(shopify)\/(.*)\/([a-zA-Z0-9]+)/i;
const base64RegExp = /^(?:[A-Za-z0-9+\\/]{4})*(?:[A-Za-z0-9+\\/]{2}(==)?|[A-Za-z0-9+\\/]{3}=?)?$/;
const isNumerical = (input) => !isNaN(Number(input));
function isStorefrontId(itemId) {
    if (typeof itemId === 'number')
        return false;
    try {
        parseStorefrontId(itemId);
        return true;
    }
    catch (_a) {
        return false;
    }
}
exports.isStorefrontId = isStorefrontId;
function isExternalId(itemId) {
    return typeof itemId === 'number';
}
exports.isExternalId = isExternalId;
function mapItemIdToProductStorefrontId(itemId) {
    if (!itemId)
        throw new Error(`id cannot be ${JSON.stringify(itemId)}`);
    if (typeof itemId === 'string' && base64RegExp.test(itemId)) {
        const [productType, externalId] = parseStorefrontId(itemId);
        if (productType === 'CartLine')
            return itemId;
        if (productType === 'Product')
            return itemId;
        return buildStorefrontId('Product', externalId);
    }
    if (typeof itemId === 'string' && !isNumerical(itemId))
        throw new Error('id is a string and not a storefrontId. Is this a product handle?');
    return buildStorefrontId('Product', Number(itemId));
}
exports.mapItemIdToProductStorefrontId = mapItemIdToProductStorefrontId;
function mapItemIdToProductVariantStorefrontId(itemId) {
    if (!itemId)
        throw new Error(`id cannot be ${JSON.stringify(itemId)}`);
    if (typeof itemId === 'string' && base64RegExp.test(itemId)) {
        const [productType, externalId] = parseStorefrontId(itemId);
        if (productType === 'CartLine')
            return itemId;
        if (productType === 'ProductVariant')
            return itemId;
        return buildStorefrontId('ProductVariant', externalId);
    }
    if (typeof itemId === 'string' && !isNumerical(itemId))
        throw new Error('id is a string and not a storefrontId. Is this a product handle?');
    return buildStorefrontId('ProductVariant', Number(itemId));
}
exports.mapItemIdToProductVariantStorefrontId = mapItemIdToProductVariantStorefrontId;
function mapItemIdToExternalId(itemId) {
    if (!itemId)
        throw new Error(`id cannot be ${JSON.stringify(itemId)}`);
    if (typeof itemId === 'number')
        return itemId;
    const [productType, externalId] = parseStorefrontId(itemId);
    if (productType === 'CartLine') {
        throw new Error('CartLine Ids cannot be converted to externalIds');
    }
    if (!isNumerical(externalId)) {
        throw new Error(`StorefrontId was parsed but the externalId is not numerical - productType: ${productType}, externalId: ${externalId}`);
    }
    return Number(externalId);
}
exports.mapItemIdToExternalId = mapItemIdToExternalId;
function buildStorefrontId(productType, externalId) {
    const gidUri = `gid://shopify/${productType}/${externalId}`;
    const encoded = btoa(gidUri);
    return encoded;
}
function parseStorefrontId(storefrontId) {
    if (storefrontId == null)
        throw new Error(`id cannot be ${storefrontId}`);
    if (!base64RegExp.test(storefrontId))
        throw new Error('id is a string and not a storefrontId. Is this a product handle?');
    const decoded = atob(storefrontId);
    const match = decoded.match(shopifyGidUriRegExp);
    if (match == undefined || typeof match[3] !== 'string') {
        throw new Error('id is a string and not a storefrontId. Is this a product handle?');
    }
    return [match[3], match[4]];
}
