"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BigCommerceInventoryHelper = void 0;
const tslib_1 = require("tslib");
/* eslint-disable eqeqeq */
/* eslint-disable unicorn/no-array-reduce */
/* eslint-disable @typescript-eslint/strict-boolean-expressions */
const api_1 = require("./api");
const get_chunks_1 = require("../../../utils/get-chunks");
const fetch_pool_1 = require("../../../utils/fetch-pool");
const checkout_error_1 = require("../../../utils/checkout-error");
const error_handler_1 = require("../../../utils/error-handler");
const validations_1 = require("../../utils/validations");
class BigCommerceInventoryHelper {
    constructor(config) {
        this.productIdByVariantId = {};
        this.defaultProductType = config.defaultProductType;
    }
    cacheProductIds(productId, variantIds) {
        if (!productId || !(variantIds === null || variantIds === void 0 ? void 0 : variantIds.length))
            return;
        for (const variantId of variantIds) {
            if (variantId) {
                this.productIdByVariantId[variantId] = productId;
            }
        }
    }
    modifyProductVariantsInventoryResponse(inventoryByProductIds, variantIds) {
        return variantIds.reduce((result, variantId) => {
            var _a;
            const product = inventoryByProductIds.find(({ variants }) => variants === null || variants === void 0 ? void 0 : variants.find(({ id }) => id === variantId));
            if (product) {
                const variant = (_a = product.variants) === null || _a === void 0 ? void 0 : _a.find(({ id }) => id === variantId);
                const variantQuantity = (variant === null || variant === void 0 ? void 0 : variant.inventory_level) || null;
                const productQuantity = product.inventory_level || null;
                const quantity = product.inventory_tracking === 'variant' ? variantQuantity : productQuantity;
                result[variantId] = {
                    availableForSale: product.availability === 'available',
                    quantity: product.inventory_tracking === 'none' ? null : quantity,
                    minOrder: product.order_quantity_minimum > 0 ? product.order_quantity_minimum : undefined,
                    maxOrder: product.order_quantity_maximum > 0 ? product.order_quantity_maximum : undefined,
                    price: (variant === null || variant === void 0 ? void 0 : variant.price) || product.price,
                };
            }
            return result;
        }, {});
    }
    modifyProductsInventoryResponse(inventoryByProductIds) {
        return inventoryByProductIds.reduce((result, product) => {
            const itemId = product.id;
            if (itemId) {
                result[itemId] = {
                    availableForSale: product.availability === 'available',
                    quantity: product.inventory_tracking === 'none' ? null : product.inventory_level,
                    minOrder: product.order_quantity_minimum > 0 ? product.order_quantity_minimum : undefined,
                    maxOrder: product.order_quantity_maximum > 0 ? product.order_quantity_maximum : undefined,
                    price: product.price,
                };
            }
            return result;
        }, {});
    }
    fetchProductIdByVariantId(variantId) {
        var _a, _b, _c, _d, _e;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const cachedProductId = this.productIdByVariantId[variantId];
            if (cachedProductId)
                return cachedProductId;
            try {
                // when we fetch productIdByVariantId, we receive all variant ids for the product and cache them
                const { data } = yield api_1.api.fetchProductIdByVariantId(variantId);
                const productId = (_b = (_a = data === null || data === void 0 ? void 0 : data.site) === null || _a === void 0 ? void 0 : _a.product) === null || _b === void 0 ? void 0 : _b.entityId;
                const variantIds = (_e = (_d = (_c = data === null || data === void 0 ? void 0 : data.site) === null || _c === void 0 ? void 0 : _c.product) === null || _d === void 0 ? void 0 : _d.variants) === null || _e === void 0 ? void 0 : _e.edges.map(({ node }) => node === null || node === void 0 ? void 0 : node.entityId);
                this.cacheProductIds(productId, variantIds);
                return productId;
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Could not fetch product id by variant id', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    fetchProductsChunkInventory(itemIds) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { data } = yield api_1.api.fetchInventoryByProductIds(itemIds);
            return this.modifyProductsInventoryResponse(data);
        });
    }
    fetchProductsChunkWithVariantsInventory(itemIds, variantIds) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { data } = yield api_1.api.fetchInventoryWithVariantsByProductIds(itemIds);
            return this.modifyProductVariantsInventoryResponse(data, variantIds);
        });
    }
    fetchProductsInventory(itemIds, productType = this.defaultProductType) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                validations_1.validateIds(itemIds);
                const ids = itemIds.map(Number).filter(Boolean);
                let idsChunks;
                let inventoryChunks;
                if (productType === 'ProductVariant') {
                    const productIds = yield fetch_pool_1.fetchPool({
                        actions: ids.map((variantId) => () => this.fetchProductIdByVariantId(variantId)),
                        concurrency: 10,
                    });
                    const filteredProductIds = productIds.filter(Boolean);
                    idsChunks = get_chunks_1.getChunks(filteredProductIds, 250);
                    inventoryChunks = yield Promise.all(idsChunks.map((idsChunk) => this.fetchProductsChunkWithVariantsInventory(idsChunk, ids)));
                }
                else {
                    idsChunks = get_chunks_1.getChunks(ids, 250);
                    inventoryChunks = yield Promise.all(idsChunks.map((idsChunk) => this.fetchProductsChunkInventory(idsChunk)));
                }
                return inventoryChunks.reduce((result, chunks) => (Object.assign(Object.assign({}, result), chunks)), {});
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Could not fetch inventory data from platform', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    isProductAvailableForSale(itemId) {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const inventory = yield this.fetchProductsInventory([itemId]);
                return Boolean((_a = inventory[itemId]) === null || _a === void 0 ? void 0 : _a.availableForSale);
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Could not fetch product availability', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    getProductQuantity(itemId) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const inventory = yield this.fetchProductsInventory([itemId]);
                const { quantity } = inventory[itemId] || {};
                if (quantity == null) {
                    console.warn('Inventory tracking is not enabled for this product');
                }
                return quantity;
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Could not fetch product quantity', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    getProductMinOrder(itemId) {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const inventory = yield this.fetchProductsInventory([itemId]);
                return (_a = inventory[itemId]) === null || _a === void 0 ? void 0 : _a.minOrder;
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Could not fetch min order', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    getProductMaxOrder(itemId) {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const inventory = yield this.fetchProductsInventory([itemId]);
                return (_a = inventory[itemId]) === null || _a === void 0 ? void 0 : _a.maxOrder;
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Could not fetch max order', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    getProductPrice(itemId) {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const inventory = yield this.fetchProductsInventory([itemId]);
                return (_a = inventory[itemId]) === null || _a === void 0 ? void 0 : _a.price;
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Could not fetch product price', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
}
exports.BigCommerceInventoryHelper = BigCommerceInventoryHelper;
