"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ShopifyInventoryHelper = void 0;
const tslib_1 = require("tslib");
/* eslint-disable @typescript-eslint/require-await */
/* eslint-disable @typescript-eslint/consistent-type-assertions */
/* eslint-disable @typescript-eslint/strict-boolean-expressions */
/* eslint-disable unicorn/no-array-reduce */
const get_platform_specific_item_id_1 = require("../../../utils/get-platform-specific-item-id");
const checkout_error_1 = require("../../../utils/checkout-error");
const get_chunks_1 = require("../../../utils/get-chunks");
const error_handler_1 = require("../../../utils/error-handler");
const api_1 = require("./api");
const validations_1 = require("../../utils/validations");
class ShopifyInventoryHelper {
    constructor(config) {
        this.getStorefrontIdsMap = (itemIds, productType) => {
            return itemIds.reduce((result, itemId) => {
                const storefrontId = this.getItemIdInPlatformSpecificFormat(itemId, productType);
                result[storefrontId] = itemId;
                return result;
            }, {});
        };
        this.modifyInventoryResponse = (inventoryNodes, storefrontIdsMap) => {
            if (!inventoryNodes)
                return {};
            return inventoryNodes.reduce((result, node) => {
                var _a, _b;
                if (node) {
                    const { id, price, availableForSale, quantityAvailable } = node;
                    const itemId = storefrontIdsMap[id];
                    if (itemId) {
                        result[itemId] = {
                            price,
                            availableForSale,
                            quantity: quantityAvailable,
                            minOrder: (_a = this.getProductFromInventory(id)) === null || _a === void 0 ? void 0 : _a.minOrder,
                            maxOrder: (_b = this.getProductFromInventory(id)) === null || _b === void 0 ? void 0 : _b.maxOrder,
                        };
                    }
                }
                return result;
            }, {});
        };
        this.client = new api_1.Client(config);
        this.inventory = config.inventory || {};
        this.defaultProductType = config.defaultProductType;
    }
    getProductFromInventory(itemId) {
        var _a, _b;
        return ((_a = this.inventory.products) === null || _a === void 0 ? void 0 : _a[itemId]) || ((_b = this.inventory.productVariants) === null || _b === void 0 ? void 0 : _b[itemId]);
    }
    getItemIdInPlatformSpecificFormat(itemId, productType) {
        return get_platform_specific_item_id_1.getPlatformSpecificItemId('shopify', itemId, productType);
    }
    fetchProductsChunkInventory(itemIds, productType = this.defaultProductType) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const storefrontIdsMap = this.getStorefrontIdsMap(itemIds, productType);
            const storefrontIds = Object.keys(storefrontIdsMap);
            const { data: { nodes }, } = yield this.client.fetchInventoryByProductIds(storefrontIds);
            return this.modifyInventoryResponse(nodes, storefrontIdsMap);
        });
    }
    fetchProductsInventory(itemIds, productType = this.defaultProductType) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                validations_1.validateIds(itemIds);
                const idsChunks = get_chunks_1.getChunks(itemIds, 250);
                const inventoryChunks = yield Promise.all(idsChunks.map((idsChunk) => this.fetchProductsChunkInventory(idsChunk, productType)));
                return inventoryChunks.reduce((result, chunks) => (Object.assign(Object.assign({}, result), chunks)), {});
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Could not fetch inventory data from platform', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    isProductAvailableForSale(itemId, productType = this.defaultProductType) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const inventory = yield this.fetchProductsInventory([itemId], productType);
                if (!inventory[itemId])
                    throw 'Product not found.';
                return Boolean(inventory[itemId].availableForSale);
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Could not fetch product availability', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    getProductMinOrder(itemId, productType = this.defaultProductType) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const id = this.getItemIdInPlatformSpecificFormat(itemId, productType);
                const item = this.getProductFromInventory(id);
                return item === null || item === void 0 ? void 0 : item.minOrder;
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Could not fetch min order', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    getProductMaxOrder(itemId, productType = this.defaultProductType) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const id = this.getItemIdInPlatformSpecificFormat(itemId, productType);
                const item = this.getProductFromInventory(id);
                return item === null || item === void 0 ? void 0 : item.maxOrder;
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Could not fetch max order', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    getProductQuantity(itemId, productType = this.defaultProductType) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const inventory = yield this.fetchProductsInventory([itemId], productType);
                if (!inventory[itemId])
                    return null;
                return inventory[itemId].quantity;
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Could not fetch product quantity', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
    getProductPrice(itemId, productType = this.defaultProductType) {
        var _a;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            try {
                const inventory = yield this.fetchProductsInventory([itemId], productType);
                return (_a = inventory[itemId]) === null || _a === void 0 ? void 0 : _a.price;
            }
            catch (e) {
                const error = new checkout_error_1.CheckoutError('Could not fetch product price', e);
                error_handler_1.handleError(error);
                throw error;
            }
        });
    }
}
exports.ShopifyInventoryHelper = ShopifyInventoryHelper;
